==> graphql-web-lite-16.6.0-2.tgz: package/utilities/printSchema.js <==
var jsutils_invariant = require("../jsutils/invariant.js");

var language_blockString = require("../language/blockString.js");

var type_definition = require("../type/definition.js");

var type_directives = require("../type/directives.js");

var type_introspection = require("../type/introspection.js");

var type_scalars = require("../type/scalars.js");

var utilities_astFromValue = require("./astFromValue.js");

var shared = require("../shared.js");

function isDefinedType(type) {
  return !type_scalars.isSpecifiedScalarType(type) && !type_introspection.isIntrospectionType(type);
}

function printFilteredSchema(schema, directiveFilter, typeFilter) {
  const directives = schema.getDirectives().filter(directiveFilter);
  const types = Object.values(schema.getTypeMap()).filter(typeFilter);
  return [ printSchemaDefinition(schema), ...directives.map((directive => function printDirective(directive) {
    return printDescription(directive) + "directive @" + directive.name + printArgs(directive.args) + (directive.isRepeatable ? " repeatable" : "") + " on " + directive.locations.join(" | ");
  }(directive))), ...types.map((type => printType(type))) ].filter(Boolean).join("\n\n");
}

function printSchemaDefinition(schema) {
  if (null == schema.description && function isSchemaOfCommonNames(schema) {
    const queryType = schema.getQueryType();
    if (queryType && "Query" !== queryType.name) {
      return !1;
    }
    const mutationType = schema.getMutationType();
    if (mutationType && "Mutation" !== mutationType.name) {
      return !1;
    }
    const subscriptionType = schema.getSubscriptionType();
    if (subscriptionType && "Subscription" !== subscriptionType.name) {
      return !1;
    }
    return !0;
  }(schema)) {
    return;
  }
  const operationTypes = [];
  const queryType = schema.getQueryType();
  if (queryType) {
    operationTypes.push(`  query: ${queryType.name}`);
  }
  const mutationType = schema.getMutationType();
  if (mutationType) {
    operationTypes.push(`  mutation: ${mutationType.name}`);
  }
  const subscriptionType = schema.getSubscriptionType();
  if (subscriptionType) {
    operationTypes.push(`  subscription: ${subscriptionType.name}`);
  }
  return printDescription(schema) + `schema {\n${operationTypes.join("\n")}\n}`;
}

function printType(type) {
  if (type_definition.isScalarType(type)) {
    return function printScalar(type) {
      return printDescription(type) + `scalar ${type.name}` + function printSpecifiedByURL(scalar) {
        if (null == scalar.specifiedByURL) {
          return "";
        }
        return ` @specifiedBy(url: ${shared.print({
          kind: shared.e.STRING,
          value: scalar.specifiedByURL
        })})`;
      }(type);
    }(type);
  }
  if (type_definition.isObjectType(type)) {
    return function printObject(type) {
      return printDescription(type) + `type ${type.name}` + printImplementedInterfaces(type) + printFields(type);
    }(type);
  }
  if (type_definition.isInterfaceType(type)) {
    return function printInterface(type) {
      return printDescription(type) + `interface ${type.name}` + printImplementedInterfaces(type) + printFields(type);
    }(type);
  }
  if (type_definition.isUnionType(type)) {
    return function printUnion(type) {
      const types = type.getTypes();
      const possibleTypes = types.length ? " = " + types.join(" | ") : "";
      return printDescription(type) + "union " + type.name + possibleTypes;
    }(type);
  }
  if (type_definition.isEnumType(type)) {
    return function printEnum(type) {
      const values = type.getValues().map(((value, i) => printDescription(value, "  ", !i) + "  " + value.name + printDeprecated(value.deprecationReason)));
      return printDescription(type) + `enum ${type.name}` + printBlock(values);
    }(type);
  }
  if (type_definition.isInputObjectType(type)) {
    return function printInputObject(type) {
      const fields = Object.values(type.getFields()).map(((f, i) => printDescription(f, "  ", !i) + "  " + printInputValue(f)));
      return printDescription(type) + `input ${type.name}` + printBlock(fields);
    }(type);
  }
  jsutils_invariant.invariant(!1);
}

function printImplementedInterfaces(type) {
  const interfaces = type.getInterfaces();
  return interfaces.length ? " implements " + interfaces.map((i => i.name)).join(" & ") : "";
}

function printFields(type) {
  return printBlock(Object.values(type.getFields()).map(((f, i) => printDescription(f, "  ", !i) + "  " + f.name + printArgs(f.args, "  ") + ": " + String(f.type) + printDeprecated(f.deprecationReason))));
}

function printBlock(items) {
  return 0 !== items.length ? " {\n" + items.join("\n") + "\n}" : "";
}

function printArgs(args, indentation = "") {
  if (0 === args.length) {
    return "";
  }
  if (args.every((arg => !arg.description))) {
    return "(" + args.map(printInputValue).join(", ") + ")";
  }
  return "(\n" + args.map(((arg, i) => printDescription(arg, "  " + indentation, !i) + "  " + indentation + printInputValue(arg))).join("\n") + "\n" + indentation + ")";
}

function printInputValue(arg) {
  const defaultAST = utilities_astFromValue.astFromValue(arg.defaultValue, arg.type);
  let argDecl = arg.name + ": " + String(arg.type);
  if (defaultAST) {
    argDecl += ` = ${shared.print(defaultAST)}`;
  }
  return argDecl + printDeprecated(arg.deprecationReason);
}

function printDeprecated(reason) {
  if (null == reason) {
    return "";
  }
  if (reason !== type_directives.DEFAULT_DEPRECATION_REASON) {
    return ` @deprecated(reason: ${shared.print({
      kind: shared.e.STRING,
      value: reason
    })})`;
  }
  return " @deprecated";
}

function printDescription(def, indentation = "", firstInBlock = !0) {
  const {description} = def;
  if (null == description) {
    return "";
  }
  return (indentation && !firstInBlock ? "\n" + indentation : indentation) + shared.print({
    kind: shared.e.STRING,
    value: description,
    block: language_blockString.isPrintableAsBlockString()
  }).replace(/\n/g, "\n" + indentation) + "\n";
}

exports.printIntrospectionSchema = function printIntrospectionSchema(schema) {
  return printFilteredSchema(schema, type_directives.isSpecifiedDirective, type_introspection.isIntrospectionType);
};

exports.printSchema = function printSchema(schema) {
  return printFilteredSchema(schema, (n => !type_directives.isSpecifiedDirective(n)), isDefinedType);
};

exports.printType = printType;

==> graphql-web-lite-16.6.0-4.tgz: package/utilities/printSchema.js <==
var jsutils_invariant = require("../jsutils/invariant.js");

var language_blockString = require("../language/blockString.js");

var type_definition = require("../type/definition.js");

var type_directives = require("../type/directives.js");

var type_introspection = require("../type/introspection.js");

var type_scalars = require("../type/scalars.js");

var utilities_astFromValue = require("./astFromValue.js");

var graphql_web = require("@0no-co/graphql.web");

function isDefinedType(type) {
  return !type_scalars.isSpecifiedScalarType(type) && !type_introspection.isIntrospectionType(type);
}

function printFilteredSchema(schema, directiveFilter, typeFilter) {
  const directives = schema.getDirectives().filter(directiveFilter);
  const types = Object.values(schema.getTypeMap()).filter(typeFilter);
  return [ printSchemaDefinition(schema), ...directives.map((directive => function printDirective(directive) {
    return printDescription(directive) + "directive @" + directive.name + printArgs(directive.args) + (directive.isRepeatable ? " repeatable" : "") + " on " + directive.locations.join(" | ");
  }(directive))), ...types.map((type => printType(type))) ].filter(Boolean).join("\n\n");
}

function printSchemaDefinition(schema) {
  if (null == schema.description && function isSchemaOfCommonNames(schema) {
    const queryType = schema.getQueryType();
    if (queryType && "Query" !== queryType.name) {
      return !1;
    }
    const mutationType = schema.getMutationType();
    if (mutationType && "Mutation" !== mutationType.name) {
      return !1;
    }
    const subscriptionType = schema.getSubscriptionType();
    if (subscriptionType && "Subscription" !== subscriptionType.name) {
      return !1;
    }
    return !0;
  }(schema)) {
    return;
  }
  const operationTypes = [];
  const queryType = schema.getQueryType();
  if (queryType) {
    operationTypes.push(`  query: ${queryType.name}`);
  }
  const mutationType = schema.getMutationType();
  if (mutationType) {
    operationTypes.push(`  mutation: ${mutationType.name}`);
  }
  const subscriptionType = schema.getSubscriptionType();
  if (subscriptionType) {
    operationTypes.push(`  subscription: ${subscriptionType.name}`);
  }
  return printDescription(schema) + `schema {\n${operationTypes.join("\n")}\n}`;
}

function printType(type) {
  if (type_definition.isScalarType(type)) {
    return function printScalar(type) {
      return printDescription(type) + `scalar ${type.name}` + function printSpecifiedByURL(scalar) {
        if (null == scalar.specifiedByURL) {
          return "";
        }
        return ` @specifiedBy(url: ${graphql_web.print({
          kind: graphql_web.Kind.STRING,
          value: scalar.specifiedByURL
        })})`;
      }(type);
    }(type);
  }
  if (type_definition.isObjectType(type)) {
    return function printObject(type) {
      return printDescription(type) + `type ${type.name}` + printImplementedInterfaces(type) + printFields(type);
    }(type);
  }
  if (type_definition.isInterfaceType(type)) {
    return function printInterface(type) {
      return printDescription(type) + `interface ${type.name}` + printImplementedInterfaces(type) + printFields(type);
    }(type);
  }
  if (type_definition.isUnionType(type)) {
    return function printUnion(type) {
      const types = type.getTypes();
      const possibleTypes = types.length ? " = " + types.join(" | ") : "";
      return printDescription(type) + "union " + type.name + possibleTypes;
    }(type);
  }
  if (type_definition.isEnumType(type)) {
    return function printEnum(type) {
      const values = type.getValues().map(((value, i) => printDescription(value, "  ", !i) + "  " + value.name + printDeprecated(value.deprecationReason)));
      return printDescription(type) + `enum ${type.name}` + printBlock(values);
    }(type);
  }
  if (type_definition.isInputObjectType(type)) {
    return function printInputObject(type) {
      const fields = Object.values(type.getFields()).map(((f, i) => printDescription(f, "  ", !i) + "  " + printInputValue(f)));
      return printDescription(type) + `input ${type.name}` + printBlock(fields);
    }(type);
  }
  jsutils_invariant.invariant(!1);
}

function printImplementedInterfaces(type) {
  const interfaces = type.getInterfaces();
  return interfaces.length ? " implements " + interfaces.map((i => i.name)).join(" & ") : "";
}

function printFields(type) {
  return printBlock(Object.values(type.getFields()).map(((f, i) => printDescription(f, "  ", !i) + "  " + f.name + printArgs(f.args, "  ") + ": " + String(f.type) + printDeprecated(f.deprecationReason))));
}

function printBlock(items) {
  return 0 !== items.length ? " {\n" + items.join("\n") + "\n}" : "";
}

function printArgs(args, indentation = "") {
  if (0 === args.length) {
    return "";
  }
  if (args.every((arg => !arg.description))) {
    return "(" + args.map(printInputValue).join(", ") + ")";
  }
  return "(\n" + args.map(((arg, i) => printDescription(arg, "  " + indentation, !i) + "  " + indentation + printInputValue(arg))).join("\n") + "\n" + indentation + ")";
}

function printInputValue(arg) {
  const defaultAST = utilities_astFromValue.astFromValue(arg.defaultValue, arg.type);
  let argDecl = arg.name + ": " + String(arg.type);
  if (defaultAST) {
    argDecl += ` = ${graphql_web.print(defaultAST)}`;
  }
  return argDecl + printDeprecated(arg.deprecationReason);
}

function printDeprecated(reason) {
  if (null == reason) {
    return "";
  }
  if (reason !== type_directives.DEFAULT_DEPRECATION_REASON) {
    return ` @deprecated(reason: ${graphql_web.print({
      kind: graphql_web.Kind.STRING,
      value: reason
    })})`;
  }
  return " @deprecated";
}

function printDescription(def, indentation = "", firstInBlock = !0) {
  const {description} = def;
  if (null == description) {
    return "";
  }
  return (indentation && !firstInBlock ? "\n" + indentation : indentation) + graphql_web.print({
    kind: graphql_web.Kind.STRING,
    value: description,
    block: language_blockString.isPrintableAsBlockString()
  }).replace(/\n/g, "\n" + indentation) + "\n";
}

exports.printIntrospectionSchema = function printIntrospectionSchema(schema) {
  return printFilteredSchema(schema, type_directives.isSpecifiedDirective, type_introspection.isIntrospectionType);
};

exports.printSchema = function printSchema(schema) {
  return printFilteredSchema(schema, (n => !type_directives.isSpecifiedDirective(n)), isDefinedType);
};

exports.printType = printType;
